@@ -1,10 +1,10 @@
 /* LDrawIni.c  Routines for finding LDrawDir and SearchDirs
-Read more on http://www.ldraw.org
+Read more on https://library.ldraw.org
 
 If you make any changes to this file, please contact me, and I'll probably
 adopt the changes for the benefit of other users.
 
-Copyright (c) 2004-2008  Lars C. Hassing (SP.lars@AM.hassings.dk)
+Copyright (C) 2004-2008  Lars C. Hassing (SP.lars@AM.hassings.dk)
 
 Permission is hereby granted, free of charge, to any person obtaining
 a copy of this software and associated documentation files (the "Software"),
@@ -46,6 +46,7 @@ If not, I don't know who wrote it.
 080910 lch Try typical locations for LDrawDir and see if they have P and PARTS
 080915 lch Added LDrawDirOrigin and SearchDirsOrigin
 160801 tds Added "ldraw\unofficial\" to GetDefaultLDrawSearch (for LPub3D)
+191009 tds Fix LDrawIni to work with UTF-8 paths.
 ******************************************************************************/
 
 /*
@@ -75,6 +76,7 @@ sprintf stat strcat strchr strcmp strcpy strdup strlen strncmp strncpy ungetc
 
 /* Preprocessor flags:
  _WIN32      VC++
+ WIN_UTF8_PATHS Paths are UTF-8, and build is Windows
  __TURBOC__  Borland TurboC
  __linux__   RedHat7.3 gcc -E -dM reveals: __ELF__ __i386 __i386__ i386
                  __i586 __i586__ i586 __linux __linux__ linux
@@ -85,11 +87,16 @@ sprintf stat strcat strchr strcmp strcpy strdup strlen strncmp strncpy ungetc
                  __GNUC__=3 __MACH__=1 __STDC__=1
 */
 /* Naming refers to Windows platform */
+#if defined(_WIN32)
+#define WIN_UTF8_PATHS
+#endif
+#ifdef WIN_UTF8_PATHS
+#include <Windows.h>
+#include <Shlwapi.h>
+#endif
 #if defined(_WIN32) || defined(__TURBOC__)
 // Disable warning message C4996: 'strcpy': This function or variable may be unsafe. Consider using strcpy_s instead.
-#ifdef Q_OS_MAC
 #pragma warning( disable : 4996 )
-#endif
 #define BACKSLASH_CHAR '\\'
 #define BACKSLASH_STRING "\\"
 #define SLASH_CHAR '/'
@@ -143,164 +150,199 @@ struct LDrawIniS *LDrawIniGet(const char *LDrawDir,
                               const char *LDrawDirOrigin,
                               int *ErrorCode)
 {
-  struct LDrawIniS *LDrawIni;
-  const char    *e;
-  char           Str[MAX_PATH_LENGTH];
-  int            i;
-  char           Key[40];
-  struct LDrawIniPrivateDataS *pd;
-  char           IniFile[MAX_PATH_LENGTH];
+   struct LDrawIniS *LDrawIni;
+   const char    *e;
+   char           Str[MAX_PATH_LENGTH];
+   int            i;
+   char           Key[40];
+   struct LDrawIniPrivateDataS *pd;
+   char           IniFile[MAX_PATH_LENGTH];
 
-  if (ErrorCode)
-    *ErrorCode = LDRAWINI_ERROR_OUT_OF_MEMORY;   /* Default                */
-  LDrawIni = (struct LDrawIniS *) calloc(1, sizeof(struct LDrawIniS));
-  if (!LDrawIni)
-    return NULL;
-  LDrawIni->PrivateData = (struct LDrawIniPrivateDataS *)
+   if (ErrorCode)
+      *ErrorCode = LDRAWINI_ERROR_OUT_OF_MEMORY;   /* Default                */
+   LDrawIni = (struct LDrawIniS *) calloc(1, sizeof(struct LDrawIniS));
+   if (!LDrawIni)
+      return NULL;
+   LDrawIni->PrivateData = (struct LDrawIniPrivateDataS *)
       calloc(1, sizeof(struct LDrawIniPrivateDataS));
-  if (!LDrawIni->PrivateData)
-    {
+   if (!LDrawIni->PrivateData)
+   {
       free(LDrawIni);
       return NULL;
-    }
+   }
 
-  /* Now initialize the struct by reading env var or ini files */
+   /* Now initialize the struct by reading env var or ini files */
 
-  /* LDrawDir */
+   /* LDrawDir */
 
-  e = LDrawDir;                /* Use argument if specified                 */
-  if (e)
-    {
+   e = LDrawDir;                /* Use argument if specified                 */
+   if (e)
+   {
       LDrawIni->LDrawDir = strdup(e);
       LDrawIni->LDrawDirOrigin = strdup(LDrawDirOrigin ? LDrawDirOrigin :
-                                                         "LDrawIniGet argument");
-    }
-  else
-    {
+                                        "LDrawIniGet argument");
+   }
+   else
+   {
       e = getenv("LDRAWDIR");   /* Try environment variable                  */
       if (e)
-        {
-          LDrawIni->LDrawDir = strdup(e);
-          LDrawIni->LDrawDirOrigin = strdup("LDRAWDIR environment variable");
-        }
+      {
+         LDrawIni->LDrawDir = strdup(e);
+         LDrawIni->LDrawDirOrigin = strdup("LDRAWDIR environment variable");
+      }
       else
-        {
-          /* Then try LDraw.ini files */
-          IniFile[0] = '\0';
-          if (LDrawIniReadSectionKey(LDrawIni, "LDraw", "BaseDirectory",
-                                     Str, sizeof(Str),
-                                     IniFile, sizeof(IniFile)))
+      {
+         /* Then try LDraw.ini files */
+         IniFile[0] = '\0';
+         if (LDrawIniReadSectionKey(LDrawIni, "LDraw", "BaseDirectory",
+                                    Str, sizeof(Str),
+                                    IniFile, sizeof(IniFile)))
+         {
+            LDrawIni->LDrawDir = strdup(Str);
+            LDrawIni->LDrawDirOrigin = strdup(IniFile);
+         }
+         else
+         {
+            /* Not found, see if default/typical locations has P and PARTS */
+            if (TryTypicalLDrawDirs(Str, sizeof(Str)))
             {
-              LDrawIni->LDrawDir = strdup(Str);
-              LDrawIni->LDrawDirOrigin = strdup(IniFile);
+               LDrawIni->LDrawDir = strdup(Str);
+               LDrawIni->LDrawDirOrigin = strdup("Typical location on disk");
             }
-          else
+            else
             {
-              /* Not found, see if default/typical locations has P and PARTS */
-              if (TryTypicalLDrawDirs(Str, sizeof(Str)))
-                {
-                  LDrawIni->LDrawDir = strdup(Str);
-                  LDrawIni->LDrawDirOrigin = strdup("Typical location on disk");
-                }
-              else
-                {
-                  if (ErrorCode)
-                    *ErrorCode = LDRAWINI_ERROR_LDRAWDIR_NOT_SET;
-                  free(LDrawIni);
-                  return NULL;
-                }
+               if (ErrorCode)
+                  *ErrorCode = LDRAWINI_ERROR_LDRAWDIR_NOT_SET;
+               free(LDrawIni);
+               return NULL;
             }
-        }
-    }
-  L3FixSlashes(LDrawIni->LDrawDir);
+         }
+      }
+   }
+   L3FixSlashes(LDrawIni->LDrawDir);
 
-  /* LDrawSearch, read symbolic dirs */
-  pd = LDrawIni->PrivateData;
+   /* LDrawSearch, read symbolic dirs */
+   pd = LDrawIni->PrivateData;
 
-  /* First try environment variable */
-  e = getenv("LDRAWSEARCH");
-  if (e)
-    {
+   /* First try environment variable */
+   e = getenv("LDRAWSEARCH");
+   if (e)
+   {
       i = SplitLDrawSearch(e, &pd->nSymbolicSearchDirs, &pd->SymbolicSearchDirs);
       if (!i)
-        return NULL;           /* No more memory, just give up              */
+      {
+         free(LDrawIni);
+         return NULL;           /* No more memory, just give up              */
+      }
       LDrawIni->SearchDirsOrigin = strdup("LDRAWSEARCH environment variable");
-    }
-  else
-    {
+   }
+   else
+   {
       e = getenv("LDRAWSEARCH01");
       if (e)
-        {
-          /* LDRAWSEARCH01 set, alloc room for 99 dirs */
-          pd->SymbolicSearchDirs = (char **) malloc(99 * sizeof(char *));
-          if (!pd->SymbolicSearchDirs)
+      {
+         /* LDRAWSEARCH01 set, alloc room for 99 dirs */
+         pd->SymbolicSearchDirs = (char **) malloc(99 * sizeof(char *));
+         if (!pd->SymbolicSearchDirs)
+         {
+            free(LDrawIni);
             return NULL;        /* No more memory, just give up              */
-          while (pd->nSymbolicSearchDirs <= 99)
+         }
+         while (pd->nSymbolicSearchDirs <= 99)
+         {
+            sprintf(Key, "LDRAWSEARCH%02d", pd->nSymbolicSearchDirs + 1);
+            e = getenv(Key);
+            if (!e)
+               break;
+            pd->SymbolicSearchDirs[pd->nSymbolicSearchDirs] = strdup(e);
+            if (!pd->SymbolicSearchDirs[pd->nSymbolicSearchDirs++])
             {
-              sprintf(Key, "LDRAWSEARCH%02d", pd->nSymbolicSearchDirs + 1);
-              e = getenv(Key);
-              if (!e)
-                break;
-              pd->SymbolicSearchDirs[pd->nSymbolicSearchDirs] = strdup(e);
-              if (!pd->SymbolicSearchDirs[pd->nSymbolicSearchDirs++])
-                return NULL;     /* No more memory, just give up              */
+               free(LDrawIni);
+               return NULL;     /* No more memory, just give up              */
             }
-          /* Reduce memory to those found */
-          pd->SymbolicSearchDirs =
-              (char **) realloc(pd->SymbolicSearchDirs,
-                                pd->nSymbolicSearchDirs * sizeof(char *));
-          if (!pd->SymbolicSearchDirs)
+         }
+         /* Reduce memory to those found */
+         if (pd->nSymbolicSearchDirs > 0)
+         {
+            pd->SymbolicSearchDirs =
+               (char **) realloc(pd->SymbolicSearchDirs,
+               pd->nSymbolicSearchDirs * sizeof(char *));
+         }
+         else
+         {
+            // realloc behavior is implemenation-dependent when requested
+            // size is 0.
+            pd->SymbolicSearchDirs =
+               (char **) realloc(pd->SymbolicSearchDirs,
+               1 * sizeof(char *));
+         }
+         if (!pd->SymbolicSearchDirs)
+         {
+            free(LDrawIni);
             return NULL;        /* No more memory, just give up              */
-          LDrawIni->SearchDirsOrigin =
-              strdup("LDRAWSEARCH01 etc. environment variables");
-        }
+         }
+         LDrawIni->SearchDirsOrigin =
+            strdup("LDRAWSEARCH01 etc. environment variables");
+      }
       else
-        {
-          /* Then try LDraw.ini files */
-          IniFile[0] = '\0';
-          if (LDrawIniReadSectionKey(LDrawIni, "LDrawSearch", "1",
-                                     Str, sizeof(Str),
-                                     IniFile, sizeof(IniFile)))
+      {
+         /* Then try LDraw.ini files */
+         IniFile[0] = '\0';
+         if (LDrawIniReadSectionKey(LDrawIni, "LDrawSearch", "1",
+                                    Str, sizeof(Str),
+                                    IniFile, sizeof(IniFile)))
+         {
+            /* Key "1" found, allocate room for 99 dirs */
+            pd->SymbolicSearchDirs = (char **) malloc(99 * sizeof(char *));
+            if (!pd->SymbolicSearchDirs)
             {
-              /* Key "1" found, allocate room for 99 dirs */
-              pd->SymbolicSearchDirs = (char **) malloc(99 * sizeof(char *));
-              if (!pd->SymbolicSearchDirs)
-                return NULL;     /* No more memory, just give up              */
-              while (pd->nSymbolicSearchDirs < 99)
-                {
-                  pd->SymbolicSearchDirs[pd->nSymbolicSearchDirs] = strdup(Str);
-                  if (!pd->SymbolicSearchDirs[pd->nSymbolicSearchDirs++])
-                    return NULL;  /* No more memory, just give up              */
-                  sprintf(Key, "%d", pd->nSymbolicSearchDirs + 1);
-                  /* Be sure to read all from same ini file */
-                  if (!LDrawIniReadIniFile(IniFile, "LDrawSearch", Key,
-                                           Str, sizeof(Str)))
-                    break;
-                }
-              /* Reduce memory to those found */
-              pd->SymbolicSearchDirs =
-                  (char **) realloc(pd->SymbolicSearchDirs,
-                                    pd->nSymbolicSearchDirs * sizeof(char *));
-              if (!pd->SymbolicSearchDirs)
-                return NULL;     /* No more memory, just give up              */
-              LDrawIni->SearchDirsOrigin = strdup(IniFile);
+               free(LDrawIni);
+               return NULL;     /* No more memory, just give up              */
             }
-          else
+            while (pd->nSymbolicSearchDirs < 99)
             {
-              /* Not in env, not in ini file, use default */
-              i = SplitLDrawSearch(GetDefaultLDrawSearch(),
-                                   &pd->nSymbolicSearchDirs,
-                                   &pd->SymbolicSearchDirs);
-              if (!i)
-                return NULL;     /* No more memory, just give up              */
-              LDrawIni->SearchDirsOrigin = strdup("Default");
+               pd->SymbolicSearchDirs[pd->nSymbolicSearchDirs] = strdup(Str);
+               if (!pd->SymbolicSearchDirs[pd->nSymbolicSearchDirs++])
+               {
+                  free(LDrawIni);
+                  return NULL;  /* No more memory, just give up              */
+               }
+               sprintf(Key, "%d", pd->nSymbolicSearchDirs + 1);
+               /* Be sure to read all from same ini file */
+               if (!LDrawIniReadIniFile(IniFile, "LDrawSearch", Key,
+                                        Str, sizeof(Str)))
+                  break;
             }
-        }
-    }
+            /* Reduce memory to those found */
+            pd->SymbolicSearchDirs =
+               (char **) realloc(pd->SymbolicSearchDirs,
+                                 pd->nSymbolicSearchDirs * sizeof(char *));
+            if (!pd->SymbolicSearchDirs)
+            {
+               free(LDrawIni);
+               return NULL;     /* No more memory, just give up              */
+            }
+            LDrawIni->SearchDirsOrigin = strdup(IniFile);
+         }
+         else
+         {
+            /* Not in env, not in ini file, use default */
+            i = SplitLDrawSearch(GetDefaultLDrawSearch(),
+                                 &pd->nSymbolicSearchDirs,
+                                 &pd->SymbolicSearchDirs);
+            if (!i)
+            {
+               free(LDrawIni);
+               return NULL;     /* No more memory, just give up              */
+            }
+            LDrawIni->SearchDirsOrigin = strdup("Default");
+         }
+      }
+   }
 
-  if (ErrorCode)
-    *ErrorCode = 0;
-  return LDrawIni;
+   if (ErrorCode)
+      *ErrorCode = 0;
+   return LDrawIni;
 }                               /* LDrawIniGet                               */
 
 /*
@@ -392,6 +434,7 @@ static void FreeSymbolicDirs(struct LDrawIniS * LDrawIni)
 
 static const char *GetDefaultLDrawSearch(void)
 {
+   // LPub3D Mod - add Unofficial folder
    return "<MODELDIR>"
    "|<HIDE><DEFPRIM><LDRAWDIR>\\p"
    "|<DEFPART><LDRAWDIR>\\parts"
@@ -399,6 +442,7 @@ static const char *GetDefaultLDrawSearch(void)
    "|<UNOFFIC><HIDE><DEFPRIM><LDRAWDIR>\\unofficial\\p"
    "|<UNOFFIC><DEFPART><LDRAWDIR>\\unofficial\\parts"
    "|<UNOFFIC><HIDE><DEFPRIM><LDRAWDIR>\\unofficial";
+   // LPub3D Mod End
 }
 
 /* Returns 1 if OK, 0 on error */
@@ -408,7 +452,7 @@ static int SplitLDrawSearch(const char *LDrawSearchString, int *nDirs, char ***D
    const char    *t;
    char          *Dir;
    int            n;
-   int            Len;
+   size_t         Len;
 
    /* Count number of dir separators '|' */
    for (n = 1, s = strchr(LDrawSearchString, '|'); s; s = strchr(s + 1, '|'))
@@ -486,7 +530,7 @@ int LDrawIniReadPreferences(const char *ApplicationID,
    CFTypeID       PropType;
 
    AppID = CFStringCreateWithCString(kCFAllocatorDefault, ApplicationID,
-                                     kCFStringEncodingASCII);
+                                     kCFStringEncodingUTF8);
    if (!AppID)
       return 0;
    /* CFPreferencesCopyAppValue looks in ~/Library/Preferences/org.ldraw.plist
@@ -499,7 +543,7 @@ int LDrawIniReadPreferences(const char *ApplicationID,
       if (strcmp(PrefKey, "BaseDirectory") == 0)
          PrefKey = "LDRAWDIR";  /* For backward compatibility                */
       AppleKey = CFStringCreateWithCString(kCFAllocatorDefault, PrefKey,
-                                           kCFStringEncodingASCII);
+                                           kCFStringEncodingUTF8);
       AppleStr = (CFStringRef) CFPreferencesCopyAppValue(AppleKey, AppID);
       CFRelease(AppleKey);
       if (AppleStr)
@@ -513,7 +557,7 @@ int LDrawIniReadPreferences(const char *ApplicationID,
    {
       /* Section (other than "LDraw") specified, look for dictionary */
       AppleSection = CFStringCreateWithCString(kCFAllocatorDefault, Section,
-                                               kCFStringEncodingASCII);
+                                               kCFStringEncodingUTF8);
       PropList = CFPreferencesCopyAppValue(AppleSection, AppID);
       CFRelease(AppleSection);
       if (PropList)
@@ -522,7 +566,7 @@ int LDrawIniReadPreferences(const char *ApplicationID,
          if (PropType == CFDictionaryGetTypeID())
          {
             AppleKey = CFStringCreateWithCString(kCFAllocatorDefault, Key,
-                                                 kCFStringEncodingASCII);
+                                                 kCFStringEncodingUTF8);
             AppleStr = (CFStringRef) CFDictionaryGetValue((CFDictionaryRef) PropList,
                                                           AppleKey);
             CFRelease(AppleKey);
@@ -548,8 +592,8 @@ int LDrawIniReadIniFile(const char *IniFile,
    char           Buf[400];
    FILE          *fp;
    int            InSection;
-   int            SectionLen;
-   int            KeyLen;
+   size_t         SectionLen;
+   size_t         KeyLen;
 
 #ifdef __APPLE__
    KeyLen = strlen(IniFile);
@@ -758,7 +802,7 @@ car.ldr      .
 c:\car.ldr   c:
 /car.ldr
 */
-      i = strlen(ModelPath);
+      i = (int)strlen(ModelPath);
       while (--i >= 0)
       {
          if (ModelPath[i] == '/' || ModelPath[i] == '\\')
@@ -823,12 +867,12 @@ int LDrawIniParseSymbolicSearchDir(struct LDrawSearchDirS * Result,
                                    const char *ModelDir,
                                    const char *HomeDir)
 {
-   const  char    *s;
-   const  char    *t;
+   const char    *s;
+   const char    *t;
    char          *Dir;
    int            Flags;
-   int            OldLen;
-   int            Len;
+   size_t         OldLen;
+   size_t         Len;
    char          *UnknownFlags;
    const char    *PrefixDir;
 
@@ -845,6 +889,7 @@ int LDrawIniParseSymbolicSearchDir(struct LDrawSearchDirS * Result,
    /* First parse any flags on the form <FLAG> */
    while (*s == '<')
    {
+      char *TempUnknownFlags;
       if (strncmp(s, "<SKIP>", 6) == 0)
       {
          Flags |= LDSDF_SKIP;
@@ -894,9 +939,14 @@ int LDrawIniParseSymbolicSearchDir(struct LDrawSearchDirS * Result,
          break;                 /* Hm, no matching >, use rest as Dir        */
       Len = t - s + 1;
       OldLen = UnknownFlags ? strlen(UnknownFlags) : 0;
-      UnknownFlags = (char *) realloc(UnknownFlags, OldLen + Len + 1);
-      if (!UnknownFlags)
+      TempUnknownFlags = (char *) realloc(UnknownFlags, OldLen + Len + 1);
+      if (!TempUnknownFlags)
+      {
+         if (UnknownFlags)
+            free(UnknownFlags);
          return 0;
+      }
+      UnknownFlags = TempUnknownFlags;
       memcpy(UnknownFlags + OldLen, s, Len);
       UnknownFlags[OldLen + Len] = '\0';
       s += Len;
@@ -940,10 +990,14 @@ int LDrawIniParseSymbolicSearchDir(struct LDrawSearchDirS * Result,
       return 0;                 /* ModelDir/HomeDir not applicable           */
    }
    OldLen = strlen(PrefixDir);
-   Len = t ? t - s : strlen(s); // generate warning: comparison between signed and unsigned type
+   Len = t ? t - s : strlen(s); // generates warning: comparison between signed and unsigned type
    Dir = (char *) malloc(OldLen + Len + 1 + 1); /* See AddTrailingSlash      */
    if (!Dir)
+   {
+      if (UnknownFlags)
+         free(UnknownFlags);
       return 0;
+   }
    strcpy(Dir, PrefixDir);
    memcpy(Dir + OldLen, s, Len);
    Dir[OldLen + Len] = '\0';
@@ -1021,14 +1075,33 @@ static void L3FixSlashes(register char *Path)
          *Path = BACKSLASH_CHAR;
 }
 
+static int L3IsDirHelper(char *Path)
+{
+#ifdef WIN_UTF8_PATHS
+   int pathBufSize = MultiByteToWideChar(CP_UTF8, 0, Path, -1, NULL, 0);
+   if (pathBufSize > 0)
+   {
+      LPWSTR WPath = malloc(pathBufSize * sizeof(WCHAR));
+      MultiByteToWideChar(CP_UTF8, 0, Path, -1, WPath, pathBufSize);
+      if (PathIsDirectoryW(WPath))
+      {
+         free(WPath);
+         return 1;
+      }
+      free(WPath);
+   }
+#endif
+   struct stat    Stat;
+   if (stat(Path, &Stat) == 0)
+      return (Stat.st_mode & S_IFDIR);
+   return 0;
+}
+
 static int L3IsDir(char *Path)
 {
-   struct stat    Stat;
 #ifdef _WIN32
    char           NewPath[4];
-#endif
 
-#ifdef _WIN32
    if (strlen(Path) == 2 && Path[1] == ':')
    {
       NewPath[0] = Path[0];
@@ -1038,12 +1111,12 @@ static int L3IsDir(char *Path)
       Path = NewPath;
    }
 #endif
-   if (stat(Path, &Stat) == 0)
-      return (Stat.st_mode & S_IFDIR);
+   if (L3IsDirHelper(Path))
+      return 1;
    if (gFileCaseCallback && gFileCaseCallback(Path))
    {
-      if (stat(Path, &Stat) == 0)
-         return (Stat.st_mode & S_IFDIR);
+      if (L3IsDirHelper(Path))
+         return 1;
    }
    return 0;
 }
