@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 - 2017 Trevor SANDY. All rights reserved.
+** Copyright (C) 2015 - 2025 Trevor SANDY. All rights reserved.
 **
 ** This file may be used under the terms of the
 ** GNU General Public Liceense (GPL) version 3.0
@@ -17,10 +17,9 @@
 
 #include <QDialog>
 #ifdef Q_OS_WIN
-#include <windows.h>
+#include <Windows.h>
 #endif
 
-
 namespace Ui {
 class AboutDialog;
 }
@@ -32,28 +31,27 @@ typedef BOOL (WINAPI *LPFN_ISWOW64PROCESS) (HANDLE, PBOOL);
 
 class AboutDialog : public QDialog
 {
-	Q_OBJECT
+    Q_OBJECT
 
 private slots:
-    void showReadmeDetails(bool);
+    void showChangeLogDetails(bool);
     void showCreditDetails(bool);
+    void updateChangelog ();
 
 public:
-        explicit AboutDialog(QWidget *parent);
+    explicit AboutDialog(QWidget *parent);
     QString osName();
     QPushButton *detailsButton;
     QPushButton *creditsButton;
-    QString content;
-	~AboutDialog();
+    ~AboutDialog();
 
 #ifdef Q_OS_WIN
     OsType checkOS();
 #endif
 
 private:
-	Ui::AboutDialog *ui;
+    Ui::AboutDialog *ui;
+    bool changeLogLoaded;
 
 };
-
-
 #endif // _ABOUTDIALOG_H_
