@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 - 2017 Trevor SANDY. All rights reserved.
+** Copyright (C) 2015 - 2025 Trevor SANDY. All rights reserved.
 **
 ** This file may be used under the terms of the
 ** GNU General Public Liceense (GPL) version 3.0
@@ -16,11 +16,8 @@
 #define ARCHIVEPARTS_H
 
 #include <QObject>
-#include "quazip.h"
-#include "quazipfile.h"
-#include "quazipdir.h"
 
-#include "QsLog.h"
+class QDir;
 
 class ArchiveParts : public QObject
 {
@@ -28,23 +25,25 @@ class ArchiveParts : public QObject
 
 public:
   explicit ArchiveParts(QObject *parent = 0);
-  ~ArchiveParts()
-  {}
+  ~ArchiveParts() {}
   static bool Archive(
       const QString &zipArchive,
       const QDir &dir,
             QString &result,
-      const QString &comment);
+            int &resultSeverity, /* 1=error,2=warning */
+      const QString &comment = QString(),
+            bool overwriteCustomPart = false);
 
     static void RecurseAddDir(
       const QDir &dir,
       QStringList &list);
 
-    static bool RecurseZipArchive(
+    static int ProcessedParts(const QString &dir);
+
+    static bool GetExistingArchiveFileList(
             QStringList &zipDirFileList,
-            QString &zipDirPath,
-      const QString &zipArchive,
-      const QDir &dir);
+            QStringList &validDirFiles,
+            const QString &zipArchive);
 
 public slots:
 
