@@ -2,7 +2,7 @@
 /****************************************************************************
 **
 ** Copyright (C) 2007-2009 Kevin Clague. All rights reserved.
-** Copyright (C) 2015 - 2017 Trevor SANDY. All rights reserved.
+** Copyright (C) 2015 - 2025 Trevor SANDY. All rights reserved.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
@@ -29,6 +29,9 @@
 #include <QDateTime>
 
 bool isMpd();
+bool isOlder(const QString &, const QDateTime &);
 bool isOlder(const QStringList &, const QDateTime &);
+bool modified(const QStringList &, bool);
+bool modified(const QVector<int> &, bool);
 
 #endif
