@@ -39,12 +39,7 @@
 **
 ****************************************************************************/
 
-#include "version.h"
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
-#include <QtWidgets/QApplication>
-#else
 #include <QApplication>
-#endif
 #include <QPainter>
 #include <QPainterPath>
 #include <QPixmapCache>
@@ -53,6 +48,8 @@
 
 #include "gradients.h"
 #include "hoverpoints.h"
+#include "version.h"
+#include "commonmenus.h"
 
 extern QPixmap cached(const QString &img);
 
@@ -61,6 +58,8 @@ GradientDialog::GradientDialog(QSize bgSize,QGradient *bgGradient, QDialog *pare
 {
     setWindowTitle(tr("Gradients"));
 
+    setWhatsThis(lpubWT(WT_DIALOG_GRADIENT,windowTitle()));
+
     QGroupBox *displayGroup = new QGroupBox(this);
     displayGroup->setTitle("Display");
     m_renderer = new GradientRenderer(bgSize,displayGroup);
@@ -143,7 +142,7 @@ GradientDialog::GradientDialog(QSize bgSize,QGradient *bgGradient, QDialog *pare
     connect(default3Button, SIGNAL(clicked()), this, SLOT(setDefault3()));
     connect(default4Button, SIGNAL(clicked()), this, SLOT(setDefault4()));
 
-    if (bgGradient){
+    if (bgGradient) {
         m_gradient = bgGradient;
         QTimer::singleShot(50, this, SLOT(setGradient()));
       } else {
@@ -185,9 +184,9 @@ void GradientDialog::setGradient(QGradient *bgGradient)
       m_conicalButton->animateClick();
     }
 
-  if (bgGradient->spread() == QGradient::PadSpread){
+  if (bgGradient->spread() == QGradient::PadSpread) {
       m_padSpreadButton->animateClick();
-    } else if (bgGradient->spread() == QGradient::RepeatSpread){
+    } else if (bgGradient->spread() == QGradient::RepeatSpread) {
       m_repeatSpreadButton->animateClick();
     } else {
       m_reflectSpreadButton->animateClick();
@@ -337,15 +336,18 @@ QGradient GradientRenderer::getGradient()
 
    } else if (m_gradientType == Qt::RadialGradientPattern) {
        QLineF line(pts[0], pts[1]);
-       if (line.length() > 132){
+       if (line.length() > 132) {
            line.setLength(132);
          }
        g = QRadialGradient(line.p1(),  qMin(m_size.width(), m_size.height()) / 3.0, line.p2());
 
    } else {
-       QLineF l(pts.at(0), pts.at(1));
-       qreal angle = l.angle(QLineF(0, 0, 1, 0));
-       if (l.dy() > 0)
+       QLineF l1(pts.at(0), pts.at(1));
+       QLineF l2(0, 0, 1, 0);
+       // l1.angle deprecated, use qMin(l1.angleTo(l2), l2.angleTo(l1)) instead
+       // qreal angle = l1.angle(QLineF(0, 0, 1, 0));
+       qreal angle = qMin(l1.angleTo(l2), l2.angleTo(l1));
+       if (l1.dy() > 0)
            angle = 360 - angle;
        g = QConicalGradient(pts.at(0), angle);
    }
@@ -371,9 +373,11 @@ void GradientRenderer::paint(QPainter *p)
         g = QRadialGradient(pts.at(0), qMin(width(), height()) / 3.0, pts.at(1));
 
     } else {
-        QLineF l(pts.at(0), pts.at(1));
-        qreal angle = l.angle(QLineF(0, 0, 1, 0));
-        if (l.dy() > 0)
+        QLineF l1(pts.at(0), pts.at(1));
+        //qreal angle = l1.angle(QLineF(0, 0, 1, 0));
+        QLineF l2(0, 0, 1, 0);
+        qreal angle = qMin(l1.angleTo(l2), l2.angleTo(l1));
+        if (l1.dy() > 0)
             angle = 360 - angle;
         g = QConicalGradient(pts.at(0), angle);
     }
@@ -415,7 +419,7 @@ void GradientRenderer::paintEvent(QPaintEvent *e)
 
         int o = 10;
 
-        QBrush bg = palette().brush(QPalette::Background);
+        QBrush bg = palette().brush(QPalette::Window);
         painter.fillRect(0, 0, o, o, bg);
         painter.fillRect(width() - o, 0, o, o, bg);
         painter.fillRect(0, height() - o, o, o, bg);
@@ -553,7 +557,7 @@ void GradientEditor::pointsUpdated()
     points += m_blue_shade->points();
     points += m_alpha_shade->points();
 
-    qSort(points.begin(), points.end(), x_less_than);
+    lpub_sort(points.begin(), points.end(), x_less_than);
 
     for (int i=0; i<points.size(); ++i) {
         qreal x = int(points.at(i).x());
