@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 - 2017 Trevor SANDY. All rights reserved.
+** Copyright (C) 2015 - 2025 Trevor SANDY. All rights reserved.
 **
 ** This file may be used under the terms of the
 ** GNU General Public Liceense (GPL) version 3.0
@@ -22,9 +22,10 @@
 #include "metatypes.h"
 #include "color.h"
 #include "scaledialog.h"
-#include "name.h"
+#include "declarations.h"
 #include "placementdialog.h"
 #include "commonmenus.h"
+#include "lpub.h"
 
 #include "ranges.h"
 #include "step.h"
@@ -69,70 +70,80 @@ void PageAttributeTextItem::setAttributes(
 
     setTextInteractionFlags(Qt::TextEditorInteraction);
 
+    setAcceptHoverEvents(true);
+    setFlag(QGraphicsItem::ItemIsFocusable);
     setFlag(QGraphicsItem::ItemIsMovable);
     setFlag(QGraphicsItem::ItemIsSelectable);
     setToolTip(_toolTip);
-    setZValue(1000);
+    setData(ObjectId, PageAttributeTextObj);
+    setZValue(PAGEATTRIBUTETEXT_ZVALUE_DEFAULT);
 }
 
 PageAttributeTextItem::PageAttributeTextItem(
   Page                      *_page,
   PageAttributeTextMeta     &_pageAttributeText,
-  QGraphicsItem             *_parent)
+  QGraphicsItem             *_parent):
+    isHovered(false),
+    mouseIsDown(false)
 {
   page          = _page;
 
+  int partCount = 0;
+  bool partCountOk = true;
+
   QString       toolTip;
   QString       name;
   switch(_pageAttributeText.type)
   {
   case PageTitleType:
-      name              = tr("Title");
-      toolTip           = tr("Title - click to edit, right-click to modify");
+      name    = tr("Title");
+      toolTip = tr("Title - click to edit, right-click to modify");
       break;
   case PageModelNameType:
-      name              = tr("Model ID");
-      toolTip           = tr("Model ID - click to edit, right-click to modify");
+      name    = tr("Model ID");
+      toolTip = tr("Model ID - click to edit, right-click to modify");
       break;
   case PageAuthorType:
-      name              = tr("Author");
-      toolTip           = tr("Author - click to edit, right-click to modify");
+      name    = tr("Author");
+      toolTip = tr("Author - click to edit, right-click to modify");
       break;
   case PageURLType:
-      name              = tr("URL");
-      toolTip           = tr("URL - click to edit, right-click to modify");
+      name    = tr("URL");
+      toolTip = tr("URL - click to edit, right-click to modify");
       break;
   case PageModelDescType:
-      name              = tr("Model Description");
-      toolTip           = tr("Model Description - click to edit, right-click to modify");
+      name    = tr("Model Description");
+      toolTip = tr("Model Description - click to edit, right-click to modify");
       break;
   case PagePublishDescType:
-      name              = tr("Publish Description");
-      toolTip           = tr("Publish Description - click to edit, right-click to modify");
+      name    = tr("Publish Description");
+      toolTip = tr("Publish Description - click to edit, right-click to modify");
       break;
   case PageCopyrightType:
-      name              = tr("Copyright");
-      toolTip           = tr("Copyright - click to edit, right-click to modify");
+      name    = tr("Copyright");
+      toolTip = tr("Copyright - click to edit, right-click to modify");
       break;
   case PageEmailType:
-      name              = tr("Email");
-      toolTip           = tr("Email - click to edit, right-click to modify");
+      name    = tr("Email");
+      toolTip = tr("Author Email - click to edit, right-click to modify");
       break;
   case PageDisclaimerType:
-      name              = tr("LEGO Disclaimer");
-      toolTip           = tr("Disclaimer - click to edit, right-click to modify");
+      name    = tr("LEGO Disclaimer");
+      toolTip = tr("LEGO Disclaimer - click to edit, right-click to modify");
       break;
-  case PagePiecesType:
-      name              = tr("Pieces");
-      toolTip           = tr("Pieces - click to edit, right-click to modify");
+  case PagePartsType:
+      partCount = _pageAttributeText.content.value().split(" ").first().toInt(&partCountOk);
+      partCountOk &= partCount == 1;
+      name    = partCountOk ? tr("Part") : tr("Parts");
+      toolTip = tr("Part count - click to edit, right-click to modify");
       break;
   case PagePlugType:
-      name              = tr("Plug");
-      toolTip           = tr("Plug - click to edit, right-click to modify");
+      name    = tr("Plug");
+      toolTip = tr("%1 Plug - click to edit, right-click to modify").arg(VER_PRODUCTNAME_STR);
       break;
   case PageCategoryType:
-      name              = tr("Category");
-      toolTip           = tr("Category - click to edit, right-click to modify");
+      name    = tr("Category");
+      toolTip = tr("Category - click to edit, right-click to modify");
       break;
     default:
       break;
@@ -154,64 +165,63 @@ void PageAttributeTextItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *eve
 {
   QMenu menu;
 
-  PlacementData placementData = placement.value();
-  QAction *placementAction    = commonMenus.placementMenu(menu,name,
-                                    commonMenus.naturalLanguagePlacementWhatsThis(relativeType,placementData,name));
-  QAction *fontAction         = commonMenus.fontMenu(menu,name);
-  QAction *colorAction        = commonMenus.colorMenu(menu,name);
-  QAction *marginAction       = commonMenus.marginMenu(menu,name);
+  QAction *placementAction      = lpub->getAct("placementAction.1");
+  PlacementData placementData   = placement.value();
+  placementAction->setWhatsThis(commonMenus.naturalLanguagePlacementWhatsThis(relativeType,placementData,name));
+  commonMenus.addAction(placementAction,menu);
 
-  //QAction *displayTextAction  = commonMenus.displayMenu(menu,name);
+  QAction *fontAction           = lpub->getAct("fontAction.1");
+  commonMenus.addAction(fontAction,menu);
 
-  int  onPageType;
-  if (page->coverPage && page->frontCover){
-      onPageType = FrontCoverPage;
-      //logInfo() << " On Front Cover Page";
-  }
-  else if(page->coverPage && page->backCover){
-      onPageType = BackCoverPage;
-      //logInfo() << " On Back Cover Page";
-  }
-  else {
-      onPageType = ContentPage;
-      //logInfo() << " On Content Page";
-  }
+  QAction *colorAction          = lpub->getAct("colorAction.1");
+  commonMenus.addAction(colorAction,menu);
+
+  QAction *marginAction         = lpub->getAct("marginAction.1");
+  commonMenus.addAction(marginAction,menu);
 
   //temporary until I figure a way to hide dependent attributes w/o crashing
   //TODO redesign attribute placement relativeTo scheme to allow hide w/o crash
-  QAction *displayTextAction = NULL;
-  if (onPageType == FrontCoverPage){
-      if (relativeType == PageModelNameType || relativeType == PagePublishDescType){
-          displayTextAction  = commonMenus.displayMenu(menu,name);
-        }
-  } else if (onPageType == BackCoverPage) {
-      switch(relativeType)
-      {
-      case PageTitleType:
-      case PageAuthorType:
-      case PageCopyrightType:
-      case PageURLType:
-      case PageEmailType:
-      case PageDisclaimerType:
-      case PagePlugType:
-          ;
-          break;
-      default:
-            displayTextAction  = commonMenus.displayMenu(menu,name);
-          break;
+  PageTypeEnc  pageType = ContentPage;
+  QAction *displayTextAction    = nullptr;
+  if (page->coverPage) {
+      if (page->frontCover) {
+          pageType = FrontCoverPage;
+          if (relativeType == PageModelNameType || relativeType == PagePublishDescType) {
+              displayTextAction = lpub->getAct("displayTextAction.1");
+              commonMenus.addAction(displayTextAction,menu);
+          }
+      } else if(page->backCover) {
+          pageType = BackCoverPage;
+          switch(relativeType)
+          {
+              case PageTitleType:
+              case PageAuthorType:
+              case PageCopyrightType:
+              case PageURLType:
+              case PageEmailType:
+              case PageDisclaimerType:
+              case PagePlugType:
+                  break;
+              default:
+                  displayTextAction = lpub->getAct("displayTextAction.1");
+                  commonMenus.addAction(displayTextAction,menu);
+                  break;
+          }
       }
+      //logInfo() << (page->coverPage ? "On Front Cover Page" : page->backCover ? "On Back Cover Page" : "On Content Page");
   }
 
-  QAction *selectedAction     = menu.exec(event->screenPos());
+  QAction *selectedAction = menu.exec(event->screenPos());
 
-  Where topOfSteps            = page->topOfSteps();
-  Where bottomOfSteps         = page->bottomOfSteps();
-  bool  useTop                = parentRelativeType != StepGroupType;
+  Where topOfSteps        = page->topOfSteps();
+  Where bottomOfSteps     = page->bottomOfSteps();
+  bool  useTop            = parentRelativeType != StepGroupType;
 
-  if (selectedAction == NULL) {
+  if (selectedAction == nullptr) {
     return;
   } else if (selectedAction == placementAction) {
 
+#ifdef QT_DEBUG_MODE
       logInfo() << "\nMOVE TEXT - "
                 << "\nPAGE- "
                 << (useTop ? " \nSingle-Step Page" : " \nMulti-Step Page")
@@ -231,63 +241,81 @@ void PageAttributeTextItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *eve
                 << "\nOTHER DATA -                  "
                 << " \nRelativeType:                " << RelNames[relativeType]       << " (" << relativeType << ")"
                 << " \nParentRelativeType:          " << RelNames[parentRelativeType] << " (" << parentRelativeType << ")"
-                << " \nOnPageType:                  " << (onPageType == 0 ? " \nContent Page" :
-                                                          onPageType == 1 ? " \nFront Cover Page" :
-                                                                            " \nBack Cover Page")  << " (" << onPageType << ")"
+                << " \nPageType:                    " << (pageType == ContentPage
+                                                            ? " \nContent Page"
+                                                            : pageType == FrontCoverPage
+                                                                  ? " \nFront Cover Page"
+                                                                  : pageType == BackCoverPage
+                                                                        ? " \nBack Cover Page"
+                                                                        : " \nDefault Page")
+                                                      << " (" << pageType << ")"
                                                    ;
+#endif
+      placement.setPageType(pageType);
       changePlacement(parentRelativeType,
                       relativeType,
-                     "Move " + name,
+                      tr("Move %1").arg(name),
                       topOfSteps,
                       bottomOfSteps,
                      &placement,
                       useTop,
-                      1,                        //append     - default
-                      true,                     //local      - default Possible TODO change to multiStep ? false : true (see StepNumber)
-                      true,                     //checkLocal - default
-                      onPageType);
+                      1,     //append     - default
+                      true); //local      - default Possible TODO change to multiStep ? false : true (see StepNumber)
+
 
   } else if (selectedAction == marginAction) {
-
+#ifdef QT_DEBUG_MODE
       logInfo() << "\nCHANGE MARGIN (TEXT) - "
                 << "\nPAGE- "
                 << (useTop ? " \nSingle-Step Page" : " \nMulti-Step Page")
                              ;
-      changeMargins(name + " Margins",
+#endif
+      changeMargins(tr("%1 Margins").arg(name),
                     topOfSteps,
                     bottomOfSteps,
                    &margin,
                     useTop);
 
   } else if (selectedAction == fontAction) {
+#ifdef QT_DEBUG_MODE
       logInfo() << "\nCHANGE FONT (TEXT) - "
                 << "\nPAGE- "
                 << (useTop ? " \nSingle-Step Page" : " \nMulti-Step Page")
                              ;
+#endif
       changeFont(topOfSteps,
                  bottomOfSteps,
                 &textFont,1, true,
                  useTop);
 
   } else if (selectedAction == colorAction) {
+#ifdef QT_DEBUG_MODE
       logInfo() << "\nCHANGE COLOUR (TEXT) - "
                 << "\nPAGE- "
                 << (useTop ? " \nSingle-Step Page" : " \nMulti-Step Page")
                              ;
+#endif
       changeColor(topOfSteps,
                   bottomOfSteps,
                  &textColor,1, true,
                   useTop);
 
-  } else if (selectedAction == displayTextAction){
-
+  } else if (selectedAction == displayTextAction) {
+#ifdef QT_DEBUG_MODE
       logInfo() << "\nCHANGE DISPLAY (TEXT) - "
                 << "\nPAGE- "
                 << " \nRelativeType:  " << RelNames[relativeType]       << " (" << relativeType << ")"
-                << " \nOnPageType:    " << (onPageType == 0 ? "Content Page" : onPageType == 1 ? "Front Cover Page" : "Back Cover Page")
+                << " \nOnPageType:    " << (pageType == ContentPage
+                                                ? "Content Page"
+                                                : pageType == FrontCoverPage
+                                                      ? "Front Cover Page"
+                                                      : pageType == BackCoverPage
+                                                            ? "Back Cover Page"
+                                                            : "Default Page")
+                                        << " (" << pageType << ")"
                 << (useTop ? " \nSingle-Step Page" : " \nMulti-Step Page")
                           ;
-
+#endif
       changeBool(topOfSteps,
                  bottomOfSteps,
                 &displayText,
@@ -297,6 +325,7 @@ void PageAttributeTextItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *eve
 
 void PageAttributeTextItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
 {
+  mouseIsDown = true;
   positionChanged = false;
   position = pos();
   QGraphicsItem::mousePressEvent(event);
@@ -312,6 +341,7 @@ void PageAttributeTextItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 
 void PageAttributeTextItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 {
+  mouseIsDown = false;
   QGraphicsItem::mouseReleaseEvent(event);
 
   if (isSelected() && (flags() & QGraphicsItem::ItemIsMovable/* && positionChanged */)) {
@@ -333,6 +363,7 @@ void PageAttributeTextItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
           placementData.offsets[1] += newPosition.y()/relativeToSize[1];
           placement.setValue(placementData);
 
+#ifdef QT_DEBUG_MODE
           logInfo() << "\nDRAG TEXT- "
                     << "\nPAGE- "
                     << (useTop ? " \nSingle-Step Page" : " \nMulti-Step Page")
@@ -353,7 +384,7 @@ void PageAttributeTextItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
                     << " \nRelativeType:                " << RelNames[relativeType]       << " (" << relativeType << ")"
                     << " \nParentRelativeType:          " << RelNames[parentRelativeType] << " (" << parentRelativeType << ")"
                        ;
-
+#endif
           changePlacementOffset( useTop ? topOfSteps : bottomOfSteps,
                                 &placement,
                                  relativeType);
@@ -379,7 +410,7 @@ void PageAttributeTextItem::focusOutEvent(QFocusEvent *event)
       QStringList list = toPlainText().split(rx);
 
       QStringList list2;
-      foreach (QString string, list){
+      Q_FOREACH (QString string, list) {
         string.replace("\"","\\\"");
         list2 << string;
         }
@@ -411,4 +442,26 @@ PageAttributeTextItem::PageAttributeTextItem()
   setFlag(QGraphicsItem::ItemIsSelectable,true);
 }
 
+void PageAttributeTextItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
+{
+  isHovered = !this->isSelected() && !mouseIsDown;
+  QGraphicsItem::hoverEnterEvent(event);
+}
 
+void PageAttributeTextItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
+{
+  isHovered = false;
+  QGraphicsItem::hoverLeaveEvent(event);
+}
+
+void PageAttributeTextItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
+{
+  QPen pen;
+  pen.setColor(isHovered ? QColor(Preferences::sceneGuideColor) : Qt::black);
+  pen.setWidth(0/*cosmetic*/);
+  pen.setStyle(isHovered ? Qt::PenStyle(Preferences::sceneGuidesLine) : Qt::NoPen);
+  painter->setPen(pen);
+  painter->setBrush(Qt::transparent);
+  painter->drawRect(this->boundingRect());
+  QGraphicsTextItem::paint(painter,option,widget);
+}
