@@ -2,7 +2,7 @@
 /****************************************************************************
 **
 ** Copyright (C) 2007-2009 Kevin Clague. All rights reserved.
-** Copyright (C) 2015 - 2017 Trevor SANDY. All rights reserved.
+** Copyright (C) 2015 - 2025 Trevor SANDY. All rights reserved.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
@@ -24,19 +24,32 @@
 class Paths {
 public:
 
-    static void mkdirs();
-    static void mkfadedirs();
+    static void mkDirs();
+    static void mkPovrayDir();
+    static void mkBlenderDir();
+    static void mkCustomDirs();
     static QString lpubDir;
     static QString tmpDir;
     static QString assemDir;
     static QString partsDir;
-    static QString viewerDir;
-    static QString fadePartDir;
-    static QString fadeSubDir;
-    static QString fadePrimDir;
-    static QString fadePrim8Dir;
-    static QString fadePrim48Dir;
-    static QStringList fadeDirs;
+    static QString bomDir;
+    static QString submodelDir;
+    static QString logsDir;
+    static QString htmlStepsDir;
+    static QString extrasDir;
+    static QString libraryDir;
+    static QString customDir;
+    static QString customPartDir;
+    static QString customSubDir;
+    static QString customTextureDir;
+    static QString customPrimDir;
+    static QString customPrim8Dir;
+    static QString customPrim48Dir;
+    static QString povrayRenderDir;
+    static QString blenderRenderDir;
+    static QStringList customDirs;
 };
 
+extern Paths paths;
+
 #endif
