@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 - 2017 Trevor SANDY. All rights reserved.
+** Copyright (C) 2015 - 2025 Trevor SANDY. All rights reserved.
 **
 ** This file may be used under the terms of the
 ** GNU General Public Liceense (GPL) version 3.0
@@ -15,8 +15,10 @@
 #ifndef PLISUBSTITUTEPARTS_H
 #define PLISUBSTITUTEPARTS_H
 
+#include <QMap>
 #include <QString>
 #include <QStringList>
+class LPubMessages;
 
 class PliSubstituteParts
 {
@@ -26,6 +28,8 @@ class PliSubstituteParts
     static QMap<QString, QString>               substituteParts;
   public:
     PliSubstituteParts();
+    static bool exportSubstitutePartsHeader();
+    static bool overwriteFile(const QString &file);
     static const bool &hasSubstitutePart(QString part);
     static const bool &getSubstitutePart(QString &part);
 };
